@@ -11,8 +11,9 @@ use warnings;
 use Carp;
 use English qw(-no_match_vars);
 use Text::Sass::Expr;
+use Data::Dumper;
 
-our $VERSION = q[0.3];
+our $VERSION = q[0.4];
 
 sub new {
   my ($class, $ref) = @_;
@@ -32,9 +33,10 @@ sub css2sass {
     $self = $self->new;
   }
 
-  my $stash = {};
-  $self->_parse_css($str, $stash);
-  return $self->_stash2sass($stash);
+  my $symbols = {};
+  my $stash   = {};
+  $self->_parse_css($str, $stash, $symbols);
+  return $self->_stash2sass($stash, $symbols);
 }
 
 sub sass2css {
@@ -44,14 +46,16 @@ sub sass2css {
     $self = $self->new;
   }
 
-  my $stash = {};
-  $self->_parse_sass($str, $stash);
-#  use Data::Dumper; carp Dumper($stash);
-  return $self->_stash2css($stash);
+  my $symbols = {};
+  my $stash   = {};
+  $self->_parse_sass($str, $stash, $symbols);
+#  use Data::Dumper; carp 'STASH ' . Dumper($stash);
+#  use Data::Dumper; carp 'SYMBOLS ' . Dumper($symbols);
+  return $self->_stash2css($stash, $symbols);
 }
 
 sub _parse_sass {
-  my ($self, $str, $substash) = @_;
+  my ($self, $str, $substash, $symbols) = @_;
 
   my $groups = [split /\n\s*?\n/smx, $str];
 
@@ -60,31 +64,102 @@ sub _parse_sass {
 
     while(my $line = shift @lines) {
       #########
-      # !x = y
+      # !x = y   variable declarations
       #
-      $line =~ s{^!(\S+)\s*=\s*(.*)$}{
-        $substash->{__variables}->{$1} = $2;
+      $line =~ s{^!(\S+)\s*=\s*(.*?)$}{
+        $symbols->{variables}->{$1} = $2;
         q[];
       }smxegi;
 
+      #########
+      # =x           and   =x(!var)
+      #   bla                bla
+      #
+      # mixin declaration
+      #
+      $line =~ s{^=(.*?)$}{
+        my $mixin_stash = {};
+        my $remaining   = join "\n", @lines;
+        @lines          = ();
+        my $proto       = $1;
+        my ($func)      = $1 =~ /^([^(]+)/smx;
+
+        #########
+        # mixins are interpolated later, so we just store the string here
+        #
+        $symbols->{mixins}->{$func} = "$proto\n$remaining\n";
+#carp qq[STASHED MIXIN $func str=$symbols->{mixins}->{$func}];
+        q[];
+      }smxegi;
+
+      #########
+      # static +mixin
+      #
+      $line =~ s{^[+]([^(]+)$}{
+        my $func      = $1;
+        my $mixin_str = $symbols->{mixins}->{$func};
+
+        $mixin_str    =~ s/^.*?\n//smx;
+        my $result    = {};
+        $self->_parse_sass($mixin_str, $result, $symbols);
+        my $mixin_tag = (keys %{$result})[0];
+        $substash->{$mixin_tag} = (values %{$result})[0];
+#carp qq[STATIC MIXIN func=$func tag=$mixin_tag str=$mixin_str results=\n].Dumper($substash->{$mixin_tag});
+
+        q[];
+      }smxegi;
+
+      #########
+      # interpolated +mixin(value)
+      #
+      $line =~ s{^[+](.*?)[(](.*?)[)]$}{
+        my ($func, $argstr) = ($1, $2);
+        my $mixin_str  = $symbols->{mixins}->{$func};
+
+        my $subsymbols = $symbols; # todo: correct scoping - is better as {%{$symbols}}
+        my $values     = [split /\s*,\s*/smx, $argstr];
+        my ($varstr)   = $mixin_str =~ /^.*?[(](.*?)[)]/smx;
+        my $vars       = [split /\s*,\s*/smx, $varstr];
+#carp q[VALUES = ].Dumper($values);
+#carp q[VARS   = ].Dumper($vars);
+
+        for my $var (@{$vars}) {
+          $var =~ s/^!//smx;
+#          carp qq[VAR=$var VAL=$values->[0]];
+
+          $subsymbols->{variables}->{$var} = shift @{$values};
+        }
+#carp q[SUBSYMS=] . Dumper($subsymbols);
+        $mixin_str    =~ s/^.*?\n//smx;
+        my $result    = {};
+        $self->_parse_sass($mixin_str, $result, $subsymbols);
+#carp qq[DYNAMIC: result=].Dumper($result);
+        $substash->{"+$func"} = $result;
+#carp qq[DYNAMIC MIXIN func=$func argstr=$argstr tag=$mixin_tag str=$mixin_str results=\n].Dumper($substash->{$mixin_tag});
+
+        q[];
+      }smxegi;
+
+
       #########
       # .class
       # #id
       # element
-      # <following content>
+      #   <following content>
       #
       $line =~ s{^([.#]?\S+)$}{
-        my $remaining = join "\n", @lines;
-        @lines = ();
+        my $remaining   = join "\n", @lines;
+        @lines          = ();
         $substash->{$1} = {};
-        $self->_parse_sass($remaining, $substash->{$1});
+        $self->_parse_sass($remaining, $substash->{$1}, $symbols);
         q[];
       }smxegi;
 
       #########
       # static & dynamic attr: value
       #
-      $line =~ s{^(\S+)\s*[:=]\s*(.*)$}{
+      $line =~ s{^(\S+)\s*[:=]\s*(.*?)$}{
+#carp qq[ATTR $1 = $2];
         $substash->{$1} = $2;
         q[];
       }smxegi;
@@ -94,19 +169,19 @@ sub _parse_sass {
       #
       $line =~ s{^[ ][ ](.*?)$}{
         my $remaining = join "\n", $1, @lines;
-        $self->_parse_sass($remaining, $substash);
-        @lines = ();
+        @lines        = ();
+        $self->_parse_sass($remaining, $substash, $symbols);
         q[];
       }smxegi;
 
       #########
       # // comments
       #
-      $line =~ s{\s*//(.*)}{
-        $substash->{__comments} ||= [];
-        push @{$substash->{__comments}}, $1;
-        q[];
-      }smxegi;
+#      $line =~ s{\s*//(.*)}{
+#        $substash->{__comments} ||= [];
+#        push @{$substash->{__comments}}, $1;
+#        q[];
+#      }smxegi;
     }
   }
 
@@ -114,7 +189,7 @@ sub _parse_sass {
 }
 
 sub _parse_css {
-  my ($self, $str, $substash) = @_;
+  my ($self, $str, $substash, $symbols) = @_;
   $str =~ s{/[*].*?[*]/}{}smxg;
   my $groups = [$str =~ /[^\n]+{.*?}/smxg];
 
@@ -148,37 +223,45 @@ sub _parse_css {
 }
 
 sub _stash2css {
-  my ($self, $stash) = @_;
+  my ($self, $stash, $symbols) = @_;
   my $groups  = [];
   my $delayed = [];
 
   for my $k (keys %{$stash}) {
-    if($k =~ /^__/smx) {
-      next;
-    }
+#    if($k =~ /^__/smx) {
+#      next;
+#    }
 
     my $str .= "$k {\n";
     for my $attr (sort keys %{$stash->{$k}}) {
       my $val = $stash->{$k}->{$attr};
 
+#      carp qq[___ATTR=$attr val=$val];
+      if($attr =~ /^[+]/smx) {
+	$attr=q[];
+      }
+
       if($attr =~ /:$/smx) {
 	my $rattr = $attr;
 	$rattr    =~ s/:$//smx;
 	for my $k2 (sort keys %{$val}) {
-	  $str .= "  $rattr-$k2: @{[$self->_expr($stash, $val->{$k2})]};\n";
+	  $str .= "  $rattr-$k2: @{[$self->_expr($stash, $symbols, $val->{$k2})]};\n";
 	}
 	next;
       }
 
       if(ref $val) {
-	push @{$delayed}, $self->_stash2css({"$k $attr" => $val});
+	my $rattr = $k . ($attr ? " $attr":q[]);
+	push @{$delayed}, $self->_stash2css({$rattr => $val}, $symbols);
 	next;
       }
 
-      $str .= "  $attr: @{[$self->_expr($stash, $val)]};\n";
+      $str .= "  $attr: @{[$self->_expr($stash, $symbols, $val)]};\n";
     }
     $str .= "}\n";
-    push @{$groups}, $str;
+    if($str !~ /[{]\s*[}]/smx) {
+      push @{$groups}, $str;
+    }
     push @{$groups}, @{$delayed};
   }
 
@@ -186,9 +269,10 @@ sub _stash2css {
 }
 
 sub _expr {
-  my ($self, $stash, $expr) = @_;
-  my $vars = $stash->{__variables} || {};
+  my ($self, $stash, $symbols, $expr) = @_;
+  my $vars = $symbols->{variables} || {};
 
+#carp qq[_EXPR VARS = ].Dumper($vars);
   $expr =~ s/!(\S+)/{$vars->{$1}||"!$1"}/smxeg;
   $expr =~ s/[#](.)(.)(.)(\b)/#${1}${1}${2}${2}${3}${3}$4/smxgi;
 
@@ -198,17 +282,18 @@ sub _expr {
   if(scalar @parts == $binary_op_parts) {
     return Text::Sass::Expr->expr(@parts);
   }
+
   return $expr;
 }
 
 sub _stash2sass {
-  my ($self, $stash) = @_;
+  my ($self, $stash, $symbols) = @_;
   my $groups = [];
 
   for my $k (keys %{$stash}) {
-    if($k =~ /^__/smx) {
-      next;
-    }
+#    if($k =~ /^__/smx) {
+#      next;
+#    }
 
     my $str .= "$k\n";
     for my $attr (sort keys %{$stash->{$k}}) {
